@@ -22,6 +22,9 @@
 import urllib, httplib, platform, threading, tempfile, traceback
 import os, sys
 from xml.etree.ElementTree import XML
+from . import Device
+
+__all__ = ['OpenPrinting']
 
 def _normalize_space (text):
     result = text.strip ()
@@ -52,7 +55,7 @@ class _QueryThread (threading.Thread):
                   (urllib.urlencode (self.parameters),
                    self.parent.language[0],
                    self.parent.language[0]))
-        print "http://%s%s?%s" % (self.parent.base_url, query_command, params)
+        self.url = "http://%s%s?%s" % (self.parent.base_url, query_command, params)
         # Send request
         result = None
         status = 1
@@ -182,16 +185,20 @@ class OpenPrinting:
                    'format': 'xml' }
         return self.webQuery(params, parse_result, (callback, user_data))
 
-    def listDrivers(self, model, callback, user_data=None):
+    def listDrivers(self, model, callback, user_data=None, extra_options=None):
         """
         Obtain a list of printer drivers.
 
-        @type model: string
-        @param model: foomatic printer model string
+        @type model: string or cupshelpers.Device
+        @param model: foomatic printer model string or a cupshelpers.Device
+        object
         @type callback: function
         @param callback: callback function, taking (integer, user_data, string)
         parameters with the first parameter being the status code, zero for
         success
+        @type extra_options: string -> string dictionary
+        @param extra_options: Additional search options, see
+        http://www.linuxfoundation.org/en/OpenPrinting/Database/Query
         @return: query handle
         """
 
@@ -209,9 +216,16 @@ class OpenPrinting:
                 #     'url': url,
                 #     'supplier': supplier,
                 #     'license': short license string e.g. GPLv2,
-                #     'licensetext': license text (HTML),
-                #     'freesoftware': Boolean,
+                #     'licensetext': license text (Plain text),
+                #     'nonfreesoftware': Boolean,
+                #     'thirdpartysupplied': Boolean,
+                #     'manufacturersupplied': Boolean,
                 #     'patents': Boolean,
+                #     'supportcontacts' (optional):
+                #       list of { 'name',
+                #                 'url',
+                #                 'level',
+                #               }
                 #     'shortdescription': short description,
                 #     'recommended': Boolean,
                 #     'functionality':
@@ -246,17 +260,40 @@ class OpenPrinting:
                     for attribute in ['name', 'url', 'supplier', 'license',
                                       'shortdescription' ]:
                         element = driver.find (attribute)
-                        if element != None:
+                        if element != None and element.text != None:
                             dict[attribute] = _normalize_space (element.text)
 
                     element = driver.find ('licensetext')
                     if element != None:
                         dict['licensetext'] = element.text
 
-                    for boolean in ['freesoftware', 'recommended',
-                                    'patents']:
+                    for boolean in ['nonfreesoftware', 'recommended',
+                                    'patents', 'thirdpartysupplied',
+                                    'manufacturersupplied']:
                         dict[boolean] = driver.find (boolean) != None
 
+                    # Make a 'freesoftware' tag for compatibility with
+                    # how the OpenPrinting API used to work (see trac
+                    # #74).
+                    dict['freesoftware'] = not dict['nonfreesoftware']
+
+                    supportcontacts = []
+                    container = driver.find ('supportcontacts')
+                    if container != None:
+                        for sc in container.findall ('supportcontact'):
+                            supportcontact = {}
+                            if sc.text != None:
+                                supportcontact['name'] = \
+                                    _normalize_space (sc.text)
+                            else:
+                                supportcontact['name'] = ""
+                            supportcontact['url'] = sc.attrib.get ('url')
+                            supportcontact['level'] = sc.attrib.get ('level')
+                            supportcontacts.append (supportcontact)
+
+                    if supportcontacts:
+                        dict['supportcontacts'] = supportcontacts
+
                     if not dict.has_key ('name') or not dict.has_key ('url'):
                         continue
 
@@ -279,11 +316,16 @@ class OpenPrinting:
                             for package in arch.findall ('package'):
                                 rpm = {}
                                 for attribute in ['realversion','version',
-                                                  'release', 'url']:
+                                                  'release', 'url', 'pkgsys']:
                                     element = package.find (attribute)
                                     if element != None:
                                         rpm[attribute] = element.text
 
+                                repositories = package.find ('repositories')
+                                if repositories != None:
+                                    for pkgsys in repositories.getchildren ():
+                                        rpm.setdefault('repositories', {})[pkgsys.tag] = pkgsys.text
+
                                 rpms[package.attrib['file']] = rpm
                             packages[arch.tag] = rpms
 
@@ -304,10 +346,12 @@ class OpenPrinting:
             except:
                 callback (1, user_data, sys.exc_info ())
 
+        if isinstance(model, Device):
+            model = model.id
+
         params = { 'type': 'drivers',
                    'moreinfo': '1',
                    'showprinterid': '1',
-                   'onlyppdfiles': '1',
                    'onlynewestdriverpackages': '1',
                    'architectures': platform.machine(),
                    'noobsoletes': '1',
@@ -315,9 +359,11 @@ class OpenPrinting:
                    'onlymanufacturer': str (self.onlymanufacturer),
                    'printer': model,
                    'format': 'xml'}
+        if extra_options:
+            params.update(extra_options)
         return self.webQuery(params, parse_result, (callback, user_data))
 
-if __name__ == "__main__":
+def _simple_gui ():
     import gtk, pprint
     gtk.gdk.threads_init ()
     class QueryApp:
